const Gio = imports.gi.Gio;
const Gtk = imports.gi.Gtk;

const ExtensionUtils = imports.misc.extensionUtils;
const Ext = ExtensionUtils.getCurrentExtension();
const Convenience = Ext.imports.convenience;
const Themes = Ext.imports.themes;

const Gettext = imports.gettext.domain("dynamic-battery");
const _ = Gettext.gettext;

let _settings;
let _row1, _row2;
let _themeListbox;

function init() {
    _settings = Convenience.getSettings();
}

function _createSection(parent, name, selectable, scrollable) {
    let outerFrame = new Gtk.Box({
        orientation: Gtk.Orientation.VERTICAL,
        margin_left: 12,
        margin_right: 12,
        margin_bottom: 12
    });

    let label = new Gtk.Label({
        label: "<b>" + name + "</b>",
        use_markup: true,
        halign: Gtk.Align.START,
        margin_top: 6
    });

    let innerFrame = new Gtk.Frame({
        margin_top: 6,
        margin_bottom: 6
    });

    let listbox = new Gtk.ListBox({
        selection_mode: selectable
            ? Gtk.SelectionMode.BROWSE
            : Gtk.SelectionMode.NONE
    });

    if (scrollable) {
        let scroll = new Gtk.ScrolledWindow({
            vexpand: true
        });
        scroll.add(listbox);
        innerFrame.add(scroll);
    } else
        innerFrame.add(listbox);

    outerFrame.add(label);
    outerFrame.add(innerFrame);
    parent.add(outerFrame);

    return listbox;
}

function _createThemeRow(list, theme) {
    let row = new Gtk.ListBoxRow();
    let label = new Gtk.Label({
        label: theme.name,
        halign: Gtk.Align.START,
        margin_top: 2,
        margin_bottom: 2,
        margin_start: 6
    });
    row.add(label);
    list.add(row);

    return row;
}

function _createChargeLevelRow(list, name, pref) {
    let row = new Gtk.ListBoxRow();
    let box = new Gtk.Box({
        homogeneous: false,
        orientation: Gtk.Orientation.HORIZONTAL,
        margin: 6,
        margin_start: 6
    });
    let label = new Gtk.Label({
        label: name,
        halign: Gtk.Align.START
    });
    let adj = new Gtk.Adjustment({
        lower: 0,
        upper: 100,
        step_increment: 1
    });
    let scale = new Gtk.Scale({
        adjustment: adj,
        halign: Gtk.Align.FILL,
        draw_value: false,
        digits: 0
    });
    scale.add_mark(10, Gtk.PositionType.BOTTOM, "");
    _settings.bind(pref, adj, "value", Gio.SettingsBindFlags.DEFAULT);

    box.pack_start(label, false, false, 0);
    box.pack_start(scale, true, true, 0);
    row.add(box);
    list.add(row);

    return row;
}

function _createSwitchRow(list, name, pref) {
    let row = new Gtk.ListBoxRow();
    let box = new Gtk.Box({
        homogeneous: false,
        orientation: Gtk.Orientation.HORIZONTAL,
        margin: 6,
        margin_start: 6
    });
    let label = new Gtk.Label({
        label: name,
        halign: Gtk.Align.START
    });
    let swithc = new Gtk.Switch();
    swithc.halign = Gtk.Align.END;
    _settings.bind(pref, swithc, "active", Gio.SettingsBindFlags.DEFAULT);

    box.pack_start(label, false, false, 0);
    box.pack_start(swithc, true, true, 0);
    row.add(box);
    list.add(row);

    return row;
}

function _headerFunc(row, before) {
    if (!before || row.get_header())
        return;

    row.set_header(new Gtk.Separator({
        orientation: Gtk.Orientation.HORIZONTAL
    }));
}

function _settingsChanged(settings, key) {
    if (key == "theme") {
        let theme = Themes.getThemeIndexByValue(settings.get_string("theme"));
        let row = _themeListbox.get_row_at_index(theme);
        _themeListbox.select_row(row);
        return;
    }
    if (key == "test-mode") {
        let testMode = settings.get_boolean("test-mode");
        _row1.sensitive = testMode;
        _row2.sensitive = testMode;
        return;
    }
}

function buildPrefsWidget() {
    let grid = new Gtk.Grid({
        column_homogeneous: true,
        orientation: Gtk.Orientation.VERTICAL,
        margin: 12,
        margin_top: 12,
        margin_bottom: 12
    });

    _themeListbox = _createSection(grid, _("Theme"), true, true);

    for (let theme of Themes.THEMES)
        _createThemeRow(_themeListbox, theme);

    _settingsChanged(_settings, "theme");
    _themeListbox.connect("row-selected", (box, row) => {
        _settings.set_string("theme", Themes.THEMES[row.get_index()].val);
    });

    let listbox2 = _createSection(grid, _("Test mode"), false, false);
    listbox2.set_header_func(_headerFunc);

    _createSwitchRow(listbox2, _("Enable test mode"), "test-mode");
    _row1 = _createChargeLevelRow(listbox2, _("Charge level"), "test-percentage");
    _row2 = _createSwitchRow(listbox2, _("Charging"), "test-charging");

    _settingsChanged(_settings, "test-mode");
    _settings.connect("changed", _settingsChanged);

    grid.show_all();
    return grid;
}
